import type { AnyObject, ExtractAggregateFields, FlattenTypes, InferSchemaTypesRead, InferSchemaTypesWrite, MaybeArray, OptionalKeysRead, PropertyType, PropertyValue, PropertyWithValue, RequiredKeysRead, Schema, SearchQuery } from '../types/index.js';
import type { Transaction } from './transaction.js';
import { RestApiProxy } from '../api/rest-api-proxy.js';
type DBRecordInternalProps<S extends Schema = Schema> = {
    readonly __id: string;
    readonly __label: string;
    readonly __proptypes?: FlattenTypes<{
        [Key in RequiredKeysRead<S>]: S[Key]['type'];
    } & {
        [Key in OptionalKeysRead<S>]?: S[Key]['type'];
    }>;
};
export type RecordProps<S extends Schema = Schema> = S extends S ? InferSchemaTypesRead<S> : {
    [K in keyof S]?: S[K];
};
export type DBRecordInferred<S extends Schema, Q extends SearchQuery<S>> = Q extends {
    aggregate: infer A extends Record<string, any>;
} ? DBRecord<S> & ExtractAggregateFields<A> : DBRecord<S>;
export type DBRecord<S extends Schema = Schema> = FlattenTypes<DBRecordInternalProps<S> & FlattenTypes<RecordProps<S>>>;
export type DBRecordTarget = DBRecord<any> | DBRecordInstance<any> | string;
export type RelationTarget = DBRecordsArrayInstance<any> | MaybeArray<DBRecord<any>> | MaybeArray<DBRecordInstance<any>> | MaybeArray<string>;
export type Relation = {
    sourceId: string;
    sourceLabel: string;
    targetId: string;
    targetLabel: string;
    type: string;
};
export type RelationDirection = 'in' | 'out';
export type RelationOptions = {
    direction?: RelationDirection;
    type?: string;
};
export type RelationDetachOptions = {
    direction?: RelationDirection;
    typeOrTypes?: string | string[];
};
export declare class DBRecordsBatchDraft {
    label: string;
    options?: {
        returnResult?: boolean;
        suggestTypes?: boolean;
    };
    payload: MaybeArray<AnyObject>;
    constructor({ label, options, payload }: {
        label: string;
        options?: {
            returnResult?: boolean;
            suggestTypes?: boolean;
        };
        payload: AnyObject;
    });
    toJson(): {
        label: string;
        options: {
            returnResult?: boolean;
            suggestTypes?: boolean;
        } | undefined;
        payload: MaybeArray<AnyObject>;
    };
}
export declare class DBRecordDraft {
    label: string;
    properties?: Array<{
        metadata?: string;
        name: string;
        type: PropertyType;
        value: PropertyValue;
        valueSeparator?: string;
    }>;
    constructor({ label, properties }: {
        label: string;
        properties?: Array<PropertyWithValue & {
            metadata?: string;
            valueSeparator?: string;
        }>;
        relation?: RelationOptions;
    });
    toJson(): {
        label: string;
        properties: {
            metadata?: string;
            name: string;
            type: PropertyType;
            value: PropertyValue;
            valueSeparator?: string;
        }[] | undefined;
    };
}
export declare class DBRecordInstance<S extends Schema = Schema, Q extends SearchQuery<S> = SearchQuery<S>> extends RestApiProxy {
    data?: DBRecordInferred<S, Q>;
    constructor(data?: DBRecordInferred<S, Q>);
    get id(): string;
    get label(): string;
    get proptypes(): FlattenTypes<FlattenTypes<{ [Key in RequiredKeysRead<S>]: S[Key]["type"]; } & { [Key_1 in OptionalKeysRead<S>]?: S[Key_1]["type"] | undefined; }>> | undefined;
    get date(): Date;
    get timestamp(): number;
    delete(transaction?: Transaction | string): Promise<import("../index.node.js").ApiResponse<{
        message: string;
    }, Record<string, any>>>;
    update<S extends Schema = Schema>(partialData: DBRecordDraft | Partial<InferSchemaTypesWrite<S>>, transaction?: Transaction | string): Promise<DBRecordInstance<S, SearchQuery<S>>>;
    set<S extends Schema = Schema>(data: DBRecordDraft | InferSchemaTypesWrite<S>, transaction?: Transaction | string): Promise<DBRecordInstance<S, SearchQuery<S>>>;
    attach(target: RelationTarget, options?: RelationOptions, transaction?: Transaction | string): Promise<import("../index.node.js").ApiResponse<{
        message: string;
    }, Record<string, any>>>;
    detach(target: RelationTarget, options?: RelationDetachOptions, transaction?: Transaction | string): Promise<import("../index.node.js").ApiResponse<{
        message: string;
    }, Record<string, any>>>;
}
export declare class DBRecordsArrayInstance<S extends Schema = Schema, Q extends SearchQuery<S> = SearchQuery<S>> extends RestApiProxy {
    data?: DBRecordInferred<S, Q>[];
    total: number | undefined;
    searchParams?: SearchQuery<S>;
    constructor(data?: DBRecordInferred<S, Q>[], total?: number, searchParams?: SearchQuery<S>);
}
export {};
